# Camino Más Largo

¡Los organizadores de la IOI 2023 están en un problema grande! Se les olvidó planear el paseo a Ópusztaszer para el día siguiente.
Pero tal vez todavía no es demasiado tarde ....


Hay $N$ atracciones en Ópusztaszer indexadas de $0$ a $N-1$. Algunas de estas atracciones están conectadas por  **carreteras** *bidireccionales*. Cada par de atracciones están conectadas por a lo 
más una carretera. Los organizadores *no saben* cuáles pares de atracciones están conectadas por carreteras. 

Decimos que la **densidad** de una red de carreteras en Ópusztaszer es **al menos** $\delta$  si cada $3$ atracciones distintas tienen al menos $\delta$ carreteras entre ellas. En otras palabras, para cada tripleta de atracciones $(u, v, w)$ tales que $0 \le u \lt v \lt w \lt N$, entre los pares de atracciones $(u,v), (v,w)$ y $(u,w)$ al menos $\delta$ pares están conectadas por una carretera.

Los organizadores *conocen*  un entero positivo  $D$ tal que la densidad de la red de carreteras es al menos  $D$.
Note que el valor de $D$ no puede ser mayor que $3$.

Los organizadores pueden hacer  **llamadas**  telefónicas al operador telefónico de Ópusztaszer para obtener información sobre las conexiones de carreteras entre ciertas atracciones. En cada llamada se debe especificar dos arreglos no vacíos de atracciones $[A[0], \ldots, A[P-1]]$ y $[B[0], \ldots, B[R-1]]$.
Las atracciones deben ser distintas dos a dos, esto es,
* $A[i] \neq A[j]$ para cada  $i$ y $j$ tales que $0 \le i \lt j \lt P$;
* $B[i] \neq B[j]$ para cada $i$ y $j$ tales que $0 \le i \lt j \lt R$;
* $A[i] \neq B[j]$ para cada  $i$ y $j$ tales que $0 \le i \lt P$ y $0\le j \lt R$.

Para cada llamada, el operador reporta si hay una carretera conectando una atracción de $A$ con una atracción de $B$.
Esto es, el operador devuelve `true` si existen $i$ y $j$ tales que  $0 \le i \lt P$ y $0\le j \lt R$, y $A[i]$ y $B[j]$ están conectadas por una carretera. En otro caso, el informador devuelve `false`.

Un **camino** de longitud $l$ es una sucesión de atracciones *distintas*  $t[0], t[1], \ldots, t[l-1]$, donde para cada $i$ entre $0$ y $l-2$, inclusive, las atracciones $t[i]$ y $t[i+1]$ están conectados por una carretera. Un camino de longitud $l$ se llama un **camino más largo** si no existe un camino de longitud al menos $l+1$.

Su tarea es ayudar a los organizadores a encontrar el camino más largo en Ópusztaszer haciendo llamadas al operador.
## Detalles de Implementación

Debes implementar el procedimiento siguiente:

```
int[] longest_trip(int N, int D)
```

* $N$: el número de atracciones en Ópusztaszer.
* $D$: la densidad mínima garantizada para la red de carreteras.
* Este  procedimiento debe devolver un arreglo $t = [t[0], t[1], \ldots, t[l-1]]$, representando un camino más largo.
* Este procedimiento puede ser llamado **múltiples veces** en cada caso de prueba.

El procedimiento anterior puede hacer llamados al procedimiento siguiente:

```
bool are_connected(int[] A, int[] B)
```

* $A$: un arreglo no vacío de atracciones distintas.
* $B$: un arreglo no vacío de atracciones distintas.
* $A$ y $B$ deben ser disjuntos.
* Este procedimiento devuelve `true` si hay una atracción de  $A$ y una atracción de  $B$ conectadas por una carretera. En otro caso, devuelve  `false`.
* Este procedimiento puede ser llamado a lo más  $32\,640$ veces en cada ejecución de `longest_trip`, y a lo más  $150\,000$ veces en total.
* La longitud total de los arreglos  $A$ y $B$ pasados a este procedimiento en todas las ejecuciones no puede exceder  $1\,500\,000$.

El evaluador es  **no adaptativo**. Cada envío es evaluado en el mismo conjunto de casos de prueba. Esto es, los valores de  $N$ y  $D$, así como los pares de atracciones conectadas por carreteras, se fijan antes de hacer un llamado a `longest_trip`.

## Ejemplos

### Ejemplo 1


Considere un escenario en el cual  $N = 5$, $D = 1$, y las carreteras son las mostradas en la figura siguiente:

![Example1](longesttrip-ex1.png "200")

El procedimiento  `longest_trip` se llama de la siguiente manera:
```
longest_trip(5, 1)
```

El procedimiento puede hacer llamados a  `are_connected` como sigue.

Llamado                               | Pares conectados por una carretera |  Valor devuelto
:---------------------------------:|:-------------------------:|:--------------:
`are_connected([0], [1, 2, 4, 3])` | $(0,1)$ y $(0,2)$       | `true`
`are_connected([2], [0])`          | $(2,0)$                   | `true`
`are_connected([2], [3])`          | $(2,3)$                   | `true`
`are_connected([1, 0], [4, 3])`    | ninguno                    | `false`

Después de la cuarta llamada, resulta que *ninguno* de los pares $(1,4)$, $(0,4)$, $(1,3)$ y $(0,3)$ está conectado por una carretera.
Como la densidad de la red es al menos  $D = 1$, vemos que en la tripleta $(0, 3, 4)$, el par de atracciones $(3,4)$ debe estar conectado por una carretera. 
De manera similar a esto, las atracciones  $0$ y $1$ deben estar conectadas.

En este punto, se puede concluir que $t = [1, 0, 2, 3, 4]$ es un camino de longitud  $5$, y que no existe un camino de longitud mayor que  $5$.
Por lo tanto, el procedimiento  `longest_trip` puede devolver $[1, 0, 2, 3, 4]$.

Considere otro escenario en el cual  $N = 4$, $D = 1$, y las carreteras entre las atracciones son como se muestra en la siguiente figura:

![Example2](longesttrip-ex2.png "200")

El procedimiento `longest_trip` se llama de la manera siguiente:
```
longest_trip(4, 1)
```


En este escenario la longitud de un camino más largo es $2$. Por lo tanto después de unas cuantas llamadas al procedimiento `are_connected`, el procedimiento `longest_trip` puede devolver uno de   $[0, 1]$, $[1, 0]$, $[2, 3]$ o $[3, 2]$.

### Ejemplo 2

La subtarea 0 contiene un ejemplo adicional de caso de prueba con $N=256$  atracciones. Este caso de prueba se incluye en el paquete adjunto que puedes descargar del sistema de competencia. 

## Restricciones

* $3 \le N \le 256$
* La suma de  $N$ en todas las llamadas a  `longest_trip` no excede $1\,024$.
* $1 \le D \le 3$


## Subtareas

1. (5 puntos) $D = 3$
1. (10 puntos) $D = 2$
1. (25 puntos) $D = 1$. Sea $l^\star$ la longitud de un camino más largo. El procedimiento `longest_trip` no tiene que devolver un camino de longitud  $l^\star$. En vez de eso, debería devolver un camino de longitud al menos  $\left\lceil \frac{l^\star}{2} \right\rceil$.
1. (60 puntos) $D = 1$



En la subtarea 4 su puntaje se determina basado en el número de llamadas al procedimiento  `are_connected` en una sola ejecución a `longest_trip`.
Sea $q$ el número máximo de llamados en todas las ejecuciones a `longest_trip` en todos los casos de prueba de la subtarea.
Su puntaje para esta subtarea se calcula de acuerdo a la siguiente tabla:

Condición               |  Puntos
:-------------------------:|:----------------------:
$2\,750 \lt q \le 32\,640$ |  $20$
$550 \lt q \le 2\,750$     |  $30$
$400 \lt q \le 550$        |  $45$
$q \le 400$                |  $60$


Si en cualquiera de los casos de prueba, los llamados al procedimiento `are_connected` no cumplen con las restricciones descritas en Detalles de Implementación, o el arreglo devuelto por `longest_trip` no es correcto, el puntaje de su solución para esta subtarea será $0$.

## Calificador de Ejemplo

Sea $C$ el número de escenarios, esto es, el número de llamadas a `longest_trip`.
El calificador de ejemplo lee la entrada en el siguiente formato:


* linea $1$: $C$

Siguen las descripciones de $C$ escenarios.

El calificador de ejemplo lee la descripción de cada escenario en el siguiente formato:

* linea $1$: $N \; D$
* linea $1 + i$ ($1 \le i \lt N$): $U_i[0] \; U_i[1] \; \ldots \; U_i[i-1]$

Aquí, cada $U_i$ es un arreglo de longitud $i$, describiendo que parejas de atracciones están conectadas por una carretera.
Para cada $i$ y $j$ tales que $1 \le i \lt N$ y $0 \le j \lt i$:
* Si las atracciones $j$ y $i$ están conectadas por una carretera, entonces el valor de $U_i[j]$ debe ser $1$;
* Si no hay una carretera conectando las atracciones $j$ y $i$, entonces el valor de $U_i[j]$ debe ser $0$.

<!-- IMPORTANT NOTE TO TRANSLATORS: THE FOLLOWING MESSAGES (IN BACKTICKS) SHOULD NOT BE TRANSLATED -->

En cada escenario, antes de ser llamado `longest_trip`, el calificador de ejemplo verifica que la densidad de la red de carreteras sea al menos $D$.
Si la condición no se cumple, imprime el mensaje `Insufficient Density` y termina.

Si el calificador de ejemplo detecta una violación a las restricciones imprimirá `Protocol Violation: <MSG>`, donde `<MSG>` es uno de los siguientes mensajes de error:


* `invalid array`: En una llamada a `are_connected`, al menos uno de los arreglos  $A$ y $B$
    - está vacío, o
    - contiene un elemento que no es un entero entre $0$ y $N-1$, inclusive, o
    - contiene el mismo elemento al menos dos veces
* `non-disjoint arrays`: En una llamada a `are_connected`, los arreglos $A$ y $B$ no son disjuntos.
* `too many calls`: El número de llamados hechos a `are_connected` excede $32\,640$ en la ejecución actual de `longest trip`, o excede $150\,000$ en total.
* `too many elements`: El número total de atracciones enviadas a `are_connected` en todas las llamadas excede $1\,500\,000$.

En otro caso, sean $t[0], t[1], \ldots, t[l - 1]$, los elementos del arreglo devueltos por `longest_trip` en un escenario, para algún $l$ no negativo.
El calificador de ejemplo imprime tres lineas para este escenario en el siguiente formato:

* linea $1$: $l$
* linea $2$: $t[0] \; t[1] \; \ldots \; t[l-1]$
* linea $3$: El número de llamados a `are_connected` en este escenario.

Finalmente, el calificador de ejemplo imprime:

* linea $1 + 3 \cdot C$: El máximo número de llamados a `are_connected` sobre todas las llamadas a `longest_trip`
